// -*- C++ -*-
/*!
 * @file  ImageViewer.h
 * @brief 2DCDP Image Viewer component
 * @date  $Date$
 *
 * @author 株式会社 東日本計算センター
 *
 * $Id$
 */

#ifndef IMAGEVIEWER_H
#define IMAGEVIEWER_H
#include <windows.h>
#include <iostream>
#include <fstream>
#include <string>
#include <stdlib.h>
#include <windef.h>
#include <typeinfo>
#include <atlstr.h>

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="port_stub_h">
// </rtc-template>

using namespace RTC;

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc/imgproc.hpp>
#include "local_basic_type.h"
/*!
 * @class ImageViewer
 * @brief 2DCDP Image Viewer component
 *
 * [機能仕様書_画像ビューアRTC] 参照
 *
 * [機能仕様書_画像ビューアRTC] 参照
 *
 * [機能仕様書_画像ビューアRTC] 参照
 *
 */
class ImageViewer
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  ImageViewer(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~ImageViewer();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   * Initialization of Viewer RTC
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre nothing special
   * @post nothing special
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   * 
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   * Activate the Viewer RTC
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre nothing special
   * @post nothing special
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   * Deactivate the Viewer RTC
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre nothing special
   * @post nothing special
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   * Periodic task of Veiwer RTC. This task read the image file.
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre nothing special
   * @post nothing special
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * Window1の表示/非表示を切り替えを指定するパラメータ。
   * 0:非表示
   * 1:表示
   * - Name: window1_showmode 1_showmode
   * - DefaultValue: 1
   * - Unit: int
   * - Range: (0,1)
   * - Constraint: none specified
   */
  int m_1_showmode;
  /*!
   * Window2の表示/非表示を切り替えを指定するパラメータ。
   * 0:非表示
   * 1:表示
   * - Name: window2_showmode 2_showmode
   * - DefaultValue: 1
   * - Unit: int
   * - Range: (0,1)
   * - Constraint: none specified
   */
  int m_2_showmode;
  /*!
   * Window3の表示/非表示を切り替えを指定するパラメータ。
   * 0:非表示
   * 1:表示
   * - Name: window3_showmode 3_showmode
   * - DefaultValue: 1
   * - Unit: int
   * - Range: (0,1)
   * - Constraint: none specified
   */
  int m_3_showmode;
  /*!
   * Window4の表示/非表示を切り替えを指定するパラメータ。
   * 0:非表示
   * 1:表示
   * - Name: window4_showmode 4_showmode
   * - DefaultValue: 1
   * - Unit: int
   * - Range: (0,1)
   * - Constraint: none specified
   */
  int m_4_showmode;
  /*!
   * Window5の表示/非表示を切り替えを指定するパラメータ。
   * 0:非表示
   * 1:表示
   * - Name: window5_showmode 5_showmode
   * - DefaultValue: 1
   * - Unit: int
   * - Range: (0,1)
   * - Constraint: none specified
   */
  int m_5_showmode;
  /*!
   * Window7の表示/非表示を切り替えを指定するパラメータ。
   * 0:非表示
   * 1:表示
   * - Name: window7_showmode 7_showmode
   * - DefaultValue: 1
   * - Unit: int
   * - Range: (0,1)
   * - Constraint: none specified
   */
  int m_7_showmode;
  /*!
   * Window1の表示サイズの切り替えを指定するパラメータ。
   * S/M/L/XLの4段階から切り替えを行う。
   * - Name: window1_showsize 1_showsize
   * - DefaultValue: 1
   * - Unit: int
   * - Range: (0,1,2,3)
   * - Constraint: none specified
   */
  int m_1_showsize;
  /*!
   * Window2の表示サイズの切り替えを指定するパラメータ。
   * S/M/L/XLの4段階から切り替えを行う。
   * - Name: window2_showsize 2_showsize
   * - DefaultValue: 1
   * - Unit: int
   * - Range: (0,1,2,3)
   * - Constraint: none specified
   */
  int m_2_showsize;
  /*!
   * Window3の表示サイズの切り替えを指定するパラメータ。
   * S/M/L/XLの4段階から切り替えを行う。
   * - Name: window3_showsize 3_showsize
   * - DefaultValue: 1
   * - Unit: int
   * - Range: (0,1,2,3)
   * - Constraint: none specified
   */
  int m_3_showsize;
  /*!
   * Window4の表示サイズの切り替えを指定するパラメータ。
   * S/M/L/XLの4段階から切り替えを行う。
   * - Name: window4_showsize 4_showsize
   * - DefaultValue: 1
   * - Unit: int
   * - Range: (0,1,2,3)
   * - Constraint: none specified
   */
  int m_4_showsize;
  /*!
   * Window5の表示サイズの切り替えを指定するパラメータ。
   * S/M/L/XLの4段階から切り替えを行う。
   * - Name: window5_showsize 5_showsize
   * - DefaultValue: 1
   * - Unit: int
   * - Range: (0,1,2,3)
   * - Constraint: none specified
   */
  int m_5_showsize;
  /*!
   * Window7の表示サイズの切り替えを指定するパラメータ。
   * S/M/L/XLの4段階から切り替えを行う。
   * - Name: window7_showsize 7_showsize
   * - DefaultValue: 2
   * - Unit: int
   * - Range: (0,1,2,3)
   * - Constraint: none specified
   */
  int m_7_showsize;
  /*!
   * textファイルを表示するか指定する。
   * 0:Text非表示
   * 1:Text表示
   * - Name: text_showmode text_mode
   * - DefaultValue: 1
   * - Unit: -
   * - Range: (0,1)
   * - Constraint: none specified
   */
  int m_text_mode;
  /*!
   * 使用する入力ポートの切り替えを行う。
   * 0:MultiCameraImage型ポート
   * 1:CameraImage型ポート
   * - Name: imageport_switch imageport_sw
   * - DefaultValue: 0
   * - Unit: int
   * - Range: (0,1)
   * - Constraint: -
   */
  int m_imageport_sw;

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  RTC::CameraImage m_inimage1;
  /*!
   * CameraImage型のデータを取得する。
   * - Type: RTC::CameraImage
   * - Number: 7（tm/width/height/bpp/format/fDiv/pixels)
   * - Semantics: OpenRTM-aist-1.1.1/InterfaceDataTypes.idlのCamer
   *              aImage型を使用する。
   * - Unit: CameraImage型
   * - Frequency: 30Hz
   * - Operation Cycle: 30Hz
   */
  InPort<RTC::CameraImage> m_inimage1In;
  RTC::CameraImage m_inimage2;
  /*!
   * CameraImage型のデータを取得する。
   * - Type: RTC::CameraImage
   * - Number: 7（tm/width/height/bpp/format/fDiv/pixels)
   * - Semantics: OpenRTM-aist-1.1.1/InterfaceDataTypes.idlのCamer
   *              aImage型を使用する。
   * - Unit: CameraImage型
   * - Frequency: 30Hz
   * - Operation Cycle: 30Hz
   */
  InPort<RTC::CameraImage> m_inimage2In;
  RTC::CameraImage m_inimage3;
  /*!
   * CameraImage型のデータを取得する。
   * - Type: RTC::CameraImage
   * - Number: 7（tm/width/height/bpp/format/fDiv/pixels)
   * - Semantics: OpenRTM-aist-1.1.1/InterfaceDataTypes.idlのCamer
   *              aImage型を使用する。
   * - Unit: CameraImage型
   * - Frequency: 30Hz
   * - Operation Cycle: 30Hz
   */
  InPort<RTC::CameraImage> m_inimage3In;
  RTC::CameraImage m_inimage4;
  /*!
   * CameraImage型のデータを取得する。
   * - Type: RTC::CameraImage
   * - Number: 7（tm/width/height/bpp/format/fDiv/pixels)
   * - Semantics: OpenRTM-aist-1.1.1/InterfaceDataTypes.idlのCamer
   *              aImage型を使用する。
   * - Unit: CameraImage型
   * - Frequency: 30Hz
   * - Operation Cycle: 30Hz
   */
  InPort<RTC::CameraImage> m_inimage4In;
  RTC::CameraImage m_inimage5;
  /*!
   * CameraImage型のデータを取得する。
   * - Type: RTC::CameraImage
   * - Number: 7（tm/width/height/bpp/format/fDiv/pixels)
   * - Semantics: OpenRTM-aist-1.1.1/InterfaceDataTypes.idlのCamer
   *              aImage型を使用する。
   * - Unit: CameraImage型
   * - Frequency: 30Hz
   * - Operation Cycle: 30Hz
   */
  InPort<RTC::CameraImage> m_inimage5In;
  RTC::CameraImage m_inimage7;
  /*!
   * CameraImage型のデータを取得する。
   * - Type: RTC::CameraImage
   * - Number: 7（tm/width/height/bpp/format/fDiv/pixels)
   * - Semantics: OpenRTM-aist-1.1.1/InterfaceDataTypes.idlのCamer
   *              aImage型を使用する。
   * - Unit: CameraImage型
   * - Frequency: 30Hz
   * - Operation Cycle: 30Hz
   */
  InPort<RTC::CameraImage> m_inimage7In;
  RTC::TimedString m_infname;
  /*!
   * Textファイルを表示するため、File名をフルパスで取得する。
   * - Type: RTC::TimedString
   * - Number: 1
   * - Semantics: OpenRTM-aist-1.1.1/InterfaceDataTypes.idlのTimed
   *              String型を使用する。
   * - Unit: -
   * - Frequency: 30Hz
   * - Operation Cycle: 30Hz
   */
  InPort<RTC::TimedString> m_infnameIn;
  RTC::MultiCameraImages m_inmultiimage;
  /*!
   * MultiCameraImages型のデータを取得する。
   * - Type: RTC::MultiCameraImages
   * - Number: 7
   * - Semantics: OpenRTM-aist-1.1.1/InterfaceDataTypes.idlのCamer
   *              aImage型を使用する。
   * - Unit: -
   * - Frequency: 30Hz
   * - Operation Cycle: 30Hz
   */
  InPort<RTC::MultiCameraImages> m_inmultiimageIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

  const std::string strWinName1;    //application window1 name
  const std::string strWinName2;    // application window2 name 
  const std::string strWinName3;    // application window3 name 
  const std::string strWinName4;    // application window4 name 
  const std::string strWinName5;    // application window5 name 
  const std::string strWinName7;    // application window7 name 

  const uWord_Type WINDOW_SIZE_S_W = 160;    // default width(S-size) of application window to create 
  const uWord_Type WINDOW_SIZE_S_H = 120;    // default height(S-size) of application window to create 

  const uWord_Type WINDOW_SIZE_M_W = 240;    // default width(M-size) of application window to create 
  const uWord_Type WINDOW_SIZE_M_H = 180;    // default height(M-size) of application window to create 

  const uWord_Type WINDOW_SIZE_L_W = 320;    // default width(L-size) of application window to create 
  const uWord_Type WINDOW_SIZE_L_H = 240;    // default height(L-size) of application window to create 

  const uWord_Type WINDOW_SIZE_XL_W = 480;   // default width(XL-size) of application window to create 
  const uWord_Type WINDOW_SIZE_XL_H = 360;   // default height(XL-size) of application window to create 

  const uWord_Type WINDOW_MOVE_X_OFFSET = (WINDOW_SIZE_L_W + 20);    // offset of application window 
  const uWord_Type WINDOW_MOVE_Y_OFFSET = (WINDOW_SIZE_L_H + 20);    // offset of application window 

  const uWord_Type WINDOW_MOVE_Y_OFFSET2 = 210;    // offset of application window to create 

  const uWord_Type WINDOW_MOVE_X1 = 400;    // defalut x1 position of application window 
  const uWord_Type WINDOW_MOVE_X2 = (WINDOW_MOVE_X1 + WINDOW_MOVE_X_OFFSET);    // default x2 position of application window to create 
  const uWord_Type WINDOW_MOVE_X3 = (WINDOW_MOVE_X2 + WINDOW_MOVE_X_OFFSET);    // default x3 position of application window to create 

  const uWord_Type WINDOW_MOVE_Y1 = 50;    // defalut y1 position of application window 
  const uWord_Type WINDOW_MOVE_Y2 = (WINDOW_MOVE_Y1 + WINDOW_MOVE_Y_OFFSET);    // defalut y2 position of application window to create
  const uWord_Type WINDOW_MOVE_Y3 = (WINDOW_MOVE_Y2 + WINDOW_MOVE_Y_OFFSET);    // defalut y3 position of application window to create
  const uWord_Type WINDOW_MOVE_Y4 = (WINDOW_MOVE_Y1 + WINDOW_MOVE_Y_OFFSET2);    // defalut y4 position of application window to create 

  const uWord_Type IMAGE_SIZE_W_MAX = 640;    // input cameraimage data maximum width 
  const uWord_Type IMAGE_SIZE_H_MAX = 480;    // input cameraimage data maximum height 

  const uWord_Type CONF_WINDOW_SHOW_ACT = 1;    // configuration parameter "activate window" 

  const uWord_Type CONF_WINDOW_SIZE_M = 1;    // configuration parameter of size "window M-size" 
  static const uWord_Type CONF_WINDOW_SIZE_MAX = 4;    // configuration parameter maximum value 

  static const uWord_Type ACTIVE_WINDOW_MAX = 6;    // maximum number of window to create 
  
  const uWord_Type DISP_RECOMM_RES_RIGHT = 1366;
  const uWord_Type DISP_RECOMM_RES_BOTTOM = 768;
  
  std::string winname_st[ACTIVE_WINDOW_MAX];

  uWord_Type uw_window1_showmode;
  uWord_Type uw_window2_showmode;
  uWord_Type uw_window3_showmode;
  uWord_Type uw_window4_showmode;
  uWord_Type uw_window5_showmode;
  uWord_Type uw_window7_showmode;
  uWord_Type uw_text_showmode;
  uWord_Type uw_imageport_sw;

  struct winSize_st
  {
    uWord_Type uW_SizeW;    // width of window 
    uWord_Type uW_SizeH;    // height of window 
  };

  struct appWindow_st
  {
    std::string winName;    // name of window to create
	winSize_st *winSizes;   // size of window to create
	uWord_Type uW_MoveX;    // x position of window to create 
	uWord_Type uW_MoveY;    // y position of window to create
	uWord_Type uW_ConfShow;    // configuration to create window 
	uWord_Type uW_ConfSize;    // configuration to set size of window 
  };

  void readCameraImage1();    //reading cameraimage data. to view the Image
  void readCameraImage2();    //reading cameraimage data. to view the Image
  void readCameraImage3();    //reading cameraimage data. to view the Image
  void readCameraImage4();    //reading cameraimage data. to view the Image
  void readCameraImage5();    //reading cameraimage data. to view the Image
  void readCameraImage7();    //reading cameraimage data. to view the Image

  void createAppWindow();    //to create apllication window
  void showCameraImage(RTC::CameraImage, std::string);    //to view the Image
  void closeAppWindow();    //to close apllication window
  void readStringPort();
  void openTextFile();
  void closeTextFile();
  uWord_Type createCommandLine();
  uWord_Type createNotepadWindowName();
  void delMemory();
  void moveTextWindow();
  uWord_Type checkImagesize(RTC::CameraImage, std::string);
  uWord_Type checkDisplay_resolution();
  void readMultiCameraImage();    //reading cameraimage data. to view the Image
  void showMultiCameraImage(RTC::MultiCameraImages);    //to view the Image
  uWord_Type checkMultiImagesize(uWord_Type, uWord_Type, std::string);
};


extern "C"
{
  DLL_EXPORT void ImageViewerInit(RTC::Manager* manager);
};

#endif // IMAGEVIEWER_H
