﻿// -*- C++ -*-
/*!
 * @file  ImageViewer.cpp
 * @brief 2DCDP Image Viewer component
 * @date $Date$
 *
 * @author 株式会社 東日本計算センター
 *
 * $Id$
 */

#include "ImageViewer.h"

TCHAR *noteWindowName_c;
TCHAR *cmdline;
std::string txtFileName;
HANDLE notepadProcessID = NULL;
HWND notepadHandle = NULL;


// Module specification
// <rtc-template block="module_spec">
static const char* imageviewer_spec[] =
  {
    "implementation_id", "ImageViewer",
    "type_name",         "ImageViewer",
    "description",       "2DCDP Image Viewer component",
    "version",           "1.0.0",
    "vendor",            "EAC",
    "category",          "Viewer",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.1_showmode", "1",
    "conf.default.2_showmode", "1",
    "conf.default.3_showmode", "1",
    "conf.default.4_showmode", "1",
    "conf.default.5_showmode", "1",
    "conf.default.7_showmode", "1",
    "conf.default.1_showsize", "1",
    "conf.default.2_showsize", "1",
    "conf.default.3_showsize", "1",
    "conf.default.4_showsize", "1",
    "conf.default.5_showsize", "1",
    "conf.default.7_showsize", "2",
    "conf.default.text_mode", "1",
    "conf.default.imageport_sw", "0",
    // Widget
    "conf.__widget__.1_showmode", "radio",
    "conf.__widget__.2_showmode", "radio",
    "conf.__widget__.3_showmode", "radio",
    "conf.__widget__.4_showmode", "radio",
    "conf.__widget__.5_showmode", "radio",
    "conf.__widget__.7_showmode", "radio",
    "conf.__widget__.1_showsize", "radio",
    "conf.__widget__.2_showsize", "radio",
    "conf.__widget__.3_showsize", "radio",
    "conf.__widget__.4_showsize", "radio",
    "conf.__widget__.5_showsize", "radio",
    "conf.__widget__.7_showsize", "radio",
    "conf.__widget__.text_mode", "radio",
    "conf.__widget__.imageport_sw", "radio",
    // Constraints
    "conf.__constraints__.1_showmode", "(0,1)",
    "conf.__constraints__.2_showmode", "(0,1)",
    "conf.__constraints__.3_showmode", "(0,1)",
    "conf.__constraints__.4_showmode", "(0,1)",
    "conf.__constraints__.5_showmode", "(0,1)",
    "conf.__constraints__.7_showmode", "(0,1)",
    "conf.__constraints__.1_showsize", "(0,1,2,3)",
    "conf.__constraints__.2_showsize", "(0,1,2,3)",
    "conf.__constraints__.3_showsize", "(0,1,2,3)",
    "conf.__constraints__.4_showsize", "(0,1,2,3)",
    "conf.__constraints__.5_showsize", "(0,1,2,3)",
    "conf.__constraints__.7_showsize", "(0,1,2,3)",
    "conf.__constraints__.text_mode", "(0,1)",
    "conf.__constraints__.imageport_sw", "(0,1)",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
ImageViewer::ImageViewer(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_inimage1In("inimage1", m_inimage1),
    m_inimage2In("inimage2", m_inimage2),
    m_inimage3In("inimage3", m_inimage3),
    m_inimage4In("inimage4", m_inimage4),
    m_inimage5In("inimage5", m_inimage5),
    m_inimage7In("inimage7", m_inimage7),
    m_infnameIn("infname", m_infname),
    m_inmultiimageIn("inmultiimage", m_inmultiimage),
	strWinName1("入力画像1"),
    strWinName2("入力画像2"),
    strWinName3("入力投影画像"),
    strWinName4("参照投影画像"),
    strWinName5("入力復元画像"),
    strWinName7("距離差分画像")
	// </rtc-template>
{
}

/*!
 * @brief destructor
 */
ImageViewer::~ImageViewer()
{
}


/*!
 * Initialization of Viewer RTC
 */
RTC::ReturnCode_t ImageViewer::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("inimage1", m_inimage1In);
  addInPort("inimage2", m_inimage2In);
  addInPort("inimage3", m_inimage3In);
  addInPort("inimage4", m_inimage4In);
  addInPort("inimage5", m_inimage5In);
  addInPort("inimage7", m_inimage7In);
  addInPort("infname", m_infnameIn);
  addInPort("inmultiimage", m_inmultiimageIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("1_showmode", m_1_showmode, "1");
  bindParameter("2_showmode", m_2_showmode, "1");
  bindParameter("3_showmode", m_3_showmode, "1");
  bindParameter("4_showmode", m_4_showmode, "1");
  bindParameter("5_showmode", m_5_showmode, "1");
  bindParameter("7_showmode", m_7_showmode, "1");
  bindParameter("1_showsize", m_1_showsize, "1");
  bindParameter("2_showsize", m_2_showsize, "1");
  bindParameter("3_showsize", m_3_showsize, "1");
  bindParameter("4_showsize", m_4_showsize, "1");
  bindParameter("5_showsize", m_5_showsize, "1");
  bindParameter("7_showsize", m_7_showsize, "2");
  bindParameter("text_mode", m_text_mode, "1");
  bindParameter("imageport_sw", m_imageport_sw, "0");
  // </rtc-template>
  winname_st[0] = strWinName1;
  winname_st[1] = strWinName2;
  winname_st[2] = strWinName3;
  winname_st[3] = strWinName4;
  winname_st[4] = strWinName5;
  winname_st[5] = strWinName7;
  noteWindowName_c = NULL;
  cmdline = NULL;
  txtFileName = "";
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ImageViewer::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ImageViewer::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ImageViewer::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * Activate the Viewer RTC
 */

RTC::ReturnCode_t ImageViewer::onActivated(RTC::UniqueId ec_id)
{
  noteWindowName_c = NULL;
  cmdline = NULL;
  notepadProcessID = NULL;
  notepadHandle = NULL;
  txtFileName = "";

  uw_window1_showmode = m_1_showmode;
  uw_window2_showmode = m_2_showmode;
  uw_window3_showmode = m_3_showmode;
  uw_window4_showmode = m_4_showmode;
  uw_window5_showmode = m_5_showmode;
  uw_window7_showmode = m_7_showmode;
  uw_text_showmode = m_text_mode;
  uw_imageport_sw = m_imageport_sw;

  checkDisplay_resolution();
  createAppWindow();

  m_inmultiimage.images.length(ACTIVE_WINDOW_MAX);

  return RTC::RTC_OK;
}

/*!
 * Deactivate the Viewer RTC
 */

RTC::ReturnCode_t ImageViewer::onDeactivated(RTC::UniqueId ec_id)
{
  closeAppWindow();
  m_inmultiimage.images.release();
  closeTextFile();
  delMemory();
  return RTC::RTC_OK;
}

/*!
 * Periodic task of Veiwer RTC. This task read the image file.
 */

RTC::ReturnCode_t ImageViewer::onExecute(RTC::UniqueId ec_id)
{
  if (uw_imageport_sw)
  {
    readCameraImage1();
    readCameraImage2();
    readCameraImage3();
    readCameraImage4();
    readCameraImage5();
    readCameraImage7();
  }
  else
  {
    readMultiCameraImage();
  }
  cv::waitKey(10);
  readStringPort();

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ImageViewer::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ImageViewer::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ImageViewer::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ImageViewer::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ImageViewer::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
* reading cameraimage data. to view the Image.
*/
void ImageViewer::readCameraImage1()
{
	if (uw_window1_showmode == CONF_WINDOW_SHOW_ACT)
	{
		if (m_inimage1In.isNew())
		{
			m_inimage1In.read();
			showCameraImage(m_inimage1, strWinName1);
			//cv::waitKey(1);
		}
	}
}

/*!
* reading cameraimage data. to view the Image.
*/
void ImageViewer::readCameraImage2()
{
	if (uw_window2_showmode == CONF_WINDOW_SHOW_ACT)
	{
		if (m_inimage2In.isNew())
		{
			m_inimage2In.read();
			showCameraImage(m_inimage2, strWinName2);
			//cv::waitKey(1);
		}
	}
}

/*!
* reading cameraimage data. to view the Image.
*/
void ImageViewer::readCameraImage3()
{
	if (uw_window3_showmode == CONF_WINDOW_SHOW_ACT)
	{
		if (m_inimage3In.isNew())
		{
			m_inimage3In.read();
			showCameraImage(m_inimage3, strWinName3);
			//cv::waitKey(1);
		}
	}
}

/*!
* reading cameraimage data. to view the Image.
*/
void ImageViewer::readCameraImage4()
{
	if (uw_window4_showmode == CONF_WINDOW_SHOW_ACT)
	{
		if (m_inimage4In.isNew())
		{
			m_inimage4In.read();
			showCameraImage(m_inimage4, strWinName4);
			//cv::waitKey(1);
		}
	}
}

/*!
* reading cameraimage data. to view the Image.
*/
void ImageViewer::readCameraImage5()
{
	if (uw_window5_showmode == CONF_WINDOW_SHOW_ACT)
	{
		if (m_inimage5In.isNew())
		{
			m_inimage5In.read();
			showCameraImage(m_inimage5, strWinName5);
			//cv::waitKey(1);
		}
	}
}

/*!
* reading cameraimage data. to view the Image.
*/
void ImageViewer::readCameraImage7()
{
	if (uw_window7_showmode == CONF_WINDOW_SHOW_ACT)
	{
		if (m_inimage7In.isNew())
		{
			m_inimage7In.read();
			showCameraImage(m_inimage7, strWinName7);
			//cv::waitKey(1);
		}
	}
}

/*!
* opening text file.
*/
void ImageViewer::readStringPort()
{
	if (uw_text_showmode)
	{
		if (m_infnameIn.isNew())
		{
			m_infnameIn.read();
			closeTextFile();
			delMemory();
			openTextFile();
		}
	}
}

/*!
* to create apllication window.
*/
void ImageViewer::createAppWindow()
{
  winSize_st stWinSizes1[CONF_WINDOW_SIZE_MAX] =
  {
    { WINDOW_SIZE_S_W, WINDOW_SIZE_S_H },
    { WINDOW_SIZE_M_W, WINDOW_SIZE_M_H },
    { WINDOW_SIZE_L_W, WINDOW_SIZE_L_H },
	{ WINDOW_SIZE_XL_W, WINDOW_SIZE_XL_H }
  };

  appWindow_st stAppWindows[ACTIVE_WINDOW_MAX] =
  {
	  { strWinName1, stWinSizes1, WINDOW_MOVE_X1, WINDOW_MOVE_Y1, uw_window1_showmode, m_1_showsize },
	  { strWinName2, stWinSizes1, WINDOW_MOVE_X1, WINDOW_MOVE_Y2, uw_window2_showmode, m_2_showsize },
	  { strWinName3, stWinSizes1, WINDOW_MOVE_X1, WINDOW_MOVE_Y3, uw_window3_showmode, m_3_showsize },
	  { strWinName4, stWinSizes1, WINDOW_MOVE_X2, WINDOW_MOVE_Y1, uw_window4_showmode, m_4_showsize },
	  { strWinName5, stWinSizes1, WINDOW_MOVE_X2, WINDOW_MOVE_Y2, uw_window5_showmode, m_5_showsize },
	  { strWinName7, stWinSizes1, WINDOW_MOVE_X3, WINDOW_MOVE_Y4, uw_window7_showmode, m_7_showsize }
  };
  std::cout << "create window sequence bigin" << std::endl;

  uWord_Type auW_Cnt;
  uWord_Type auW_Size = CONF_WINDOW_SIZE_M;

  for (auW_Cnt = 0; auW_Cnt < ACTIVE_WINDOW_MAX; auW_Cnt++)
  {
    if (stAppWindows[auW_Cnt].uW_ConfShow == CONF_WINDOW_SHOW_ACT)
    {
      auW_Size = stAppWindows[auW_Cnt].uW_ConfSize;
	  if (auW_Size < CONF_WINDOW_SIZE_MAX)
      {
		std::cout << "create window " << "[" << stAppWindows[auW_Cnt].winName << "]" << "/size:" << auW_Size << std::endl;
		cv::namedWindow(stAppWindows[auW_Cnt].winName, cv::WINDOW_NORMAL);
		cv::resizeWindow(stAppWindows[auW_Cnt].winName, stAppWindows[auW_Cnt].winSizes[auW_Size].uW_SizeW, stAppWindows[auW_Cnt].winSizes[auW_Size].uW_SizeH);
		cv::moveWindow(stAppWindows[auW_Cnt].winName, stAppWindows[auW_Cnt].uW_MoveX, stAppWindows[auW_Cnt].uW_MoveY);
	  }
    }
    else
    {
		std::cout << "not create window " << "[" << stAppWindows[auW_Cnt].winName << "]" << std::endl;
    }
  }
  std::cout << "create window sequence end" << std::endl;
}

/*!
* to view the Image.
*/
void ImageViewer::showCameraImage(RTC::CameraImage image, std::string cam_image)
{
  if (checkImagesize(image, cam_image))
  {
    cv::Mat m_inimagebuf = cv::Mat(cv::Size(image.width, image.height), CV_8UC3);
    memcpy(m_inimagebuf.data, (void *)&(image.pixels[0]), image.pixels.length());
	cv::namedWindow(cam_image, cv::WINDOW_NORMAL);
	cv::imshow(cam_image, m_inimagebuf);
#ifdef _DEBUG
	std::cout << "width:" << image.width << std::endl;
	std::cout << "height:" << image.height << std::endl;
	std::cout << cam_image << ":rows:" << m_inimagebuf.rows << std::endl;
	std::cout << cam_image << ":cols:" << m_inimagebuf.cols << std::endl;
	std::cout << cam_image << ":channels:" << m_inimagebuf.channels() << std::endl;
	std::cout << cam_image << ":depth:" << m_inimagebuf.rows << std::endl;
	std::cout << cam_image << ":elememSize1:" << m_inimagebuf.elemSize1() << std::endl;
#endif
  }
}
/*!
* to close apllication window
*/
void ImageViewer::closeAppWindow()
{
  std::cout << "destroy all windows" << std::endl;
  cv::destroyAllWindows();
}

/*!
* opening text file.
*/
void ImageViewer::openTextFile()
{
	PROCESS_INFORMATION pi = {};
	STARTUPINFO si = { sizeof(STARTUPINFO) };

	std::cout << "open text file bigin" << std::endl;
	
	if (createCommandLine() == TRUE)
	{
		if (!CreateProcess(NULL, cmdline, NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi))
		{
			std::cerr << "create notepad process fail" << std::endl;
			notepadProcessID = NULL;
			std::cout << "cmdline free" << std::endl;
		}
		else
		{
#ifdef _DEBUG
			std::cout << "pi.hProcess=" << pi.hProcess << std::endl;
			std::cout << "pi.hThread=" << pi.hThread << std::endl;
			std::cout << "pi.dwProcessId=" << pi.dwProcessId << std::endl;
#endif
			notepadProcessID = pi.hProcess;

			if (!CloseHandle(pi.hThread))
			{
				std::cerr << "close handle error" << std::endl;
			}
			DWORD ret = WaitForSingleObject(notepadProcessID, 50);
			createNotepadWindowName();
			moveTextWindow();
		}
	}
	std::cout << "open text file end" << std::endl;
}

/*!
*
*/
void ImageViewer::moveTextWindow()
{
	notepadHandle = NULL;

	uWord_Type textSizeW = WINDOW_SIZE_M_W + 10;
	uWord_Type textSizeH = WINDOW_SIZE_M_H + 30;

	if (noteWindowName_c != NULL)
	{
		notepadHandle = FindWindow(_T("notepad"), noteWindowName_c);
	}
	if (notepadHandle != NULL)
	{
		MoveWindow(notepadHandle, WINDOW_MOVE_X2, WINDOW_MOVE_Y3, textSizeW, textSizeH, TRUE);
	}
	else
	{
		std::cerr << "find window fail" << std::endl;
	}
}
/*!
* 
*/
uWord_Type ImageViewer::createCommandLine()
{
	//RTC入力ポートから読み込んだフルパス情報を基に、commandline情報を作成
	std::string path = m_infname.data;

	if (path.find("\\") != std::string::npos)
	{
		std::replace(path.begin(), path.end(), '\\', '/');
		std::cout << "path " << path << std::endl;
	}
	std::string cmdline_str = "notepad " + path;

	//フルパス情報からファイル名を取得
	txtFileName = path.substr(path.rfind("/") + 1, path.length());
	std::cout << "text file name=" << txtFileName << std::endl;
#ifdef _DEBUG
	std::cout << "m_infname.data=" << m_infname.data << std::endl;
	std::cout << "cmdline_str=" << cmdline_str << std::endl;
#endif
	//実行CommandLine情報をstingからchar型へ変換
	//cmdline = (TCHAR *)std::malloc(strlen(cmdline_str.c_str()) + 1);
	cmdline = new TCHAR[strlen(cmdline_str.c_str()) + 1];

	if (cmdline == NULL)
	{
		std::cerr << "cmdline malloc error" << std::endl;
		return FALSE;
	}
	else
	{
		std::sprintf(cmdline, "%s", cmdline_str.c_str());
		return TRUE;
	}
}
/*!
* to close text file
*/
uWord_Type ImageViewer::createNotepadWindowName()
{
	//ファイル名からwindow名を作成
	std::string noteWindowName = txtFileName + " - メモ帳";
	std::cout << "notepad window name=" << noteWindowName << std::endl;

	//window名をstringからchar型へ変換
//	noteWindowName_c = (TCHAR *)std::malloc(strlen(noteWindowName.c_str()) + 1);
	noteWindowName_c = new TCHAR[strlen(noteWindowName.c_str()) + 1];
	if (noteWindowName_c == NULL)
	{
		std::cerr << "text file name malloc error" << std::endl;
		return FALSE;
	}
	std::sprintf(noteWindowName_c, "%s", noteWindowName.c_str());
	return TRUE;

}

/*!
* to close text file
*/
void ImageViewer::closeTextFile()
{
  std::cout << "close text file bigin" << std::endl;
  std::cout << " notepadProcessID=" << notepadProcessID << std::endl;

  if (noteWindowName_c != NULL)
  {
	  if (notepadHandle != NULL)
	{
		//notepad アプリ終了
		  PostMessage(notepadHandle, WM_CLOSE, 0, 0);
		  std::cout << " notepadHandle= " << notepadHandle<< std::endl;

		  if (!CloseHandle(notepadProcessID))
		{
			std::cerr << "handle close error" << std::endl;
			//printError("CloseHandle(childProcess)");
		}
	}
	else
	{
		std::cout << "handle=NULL" << std::endl;
	}
  }
  std::cout << "close text file end" << std::endl;
}

/*!
* deleate memory
*/
void ImageViewer::delMemory()
{
	if (cmdline != NULL)
	{
		std::cout << "cmdline del" << std::endl;
		//std::free(cmdline);
		delete[] cmdline;
		cmdline = NULL;
	}
	if (noteWindowName_c != NULL)
	{
		std::cout << "noteWindowName_c del" << std::endl;
		//std::free(noteWindowName_c);
		delete[] noteWindowName_c;
		noteWindowName_c = NULL;
	}
}

/*!
* Camera Image data size check
*/
uWord_Type ImageViewer::checkImagesize(RTC::CameraImage image, std::string cam_image)
{
  uWord_Type retVal;
  retVal = false;

  if ((image.width == 0) || (image.height == 0))
  {
    std::cerr << "[" << cam_image << "]" << "input CameraImage data is empty!" << std::endl;
  }
  else if ((image.width <= IMAGE_SIZE_W_MAX) && (image.height <= IMAGE_SIZE_H_MAX))
  {
    retVal = true;
  }
  else
  {
    std::cerr << "[" << cam_image << "]" << "input CameraImage data is oversized!" << std::endl;
  }  
  return retVal;
}

/*!
* Dislpay resolution check
*/
uWord_Type ImageViewer::checkDisplay_resolution()
{
	uWord_Type retVal;
	RECT rc;
	retVal = true;

	GetWindowRect(GetDesktopWindow(), &rc);
#ifdef _DEBUG
	std::cout << "GetWindowRect:" << rc.right << "/" << rc.bottom << std::endl;
#endif	
	if ((rc.right < DISP_RECOMM_RES_RIGHT) || (rc.bottom < DISP_RECOMM_RES_BOTTOM))
	{
		retVal = false;
		std::cout << "Display resolution is not enough." << std::endl;
		std::cout << "some Images may not display." << std::endl;
	}
	return retVal;
}

/*!
* read MultiCameraImage
*/
void ImageViewer::readMultiCameraImage()
{
	if (m_inmultiimageIn.isNew())
	{
		m_inmultiimageIn.read();
		showMultiCameraImage(m_inmultiimage);
	}
}

/*!
* show Multi Cmaera Image
*/
void ImageViewer::showMultiCameraImage(RTC::MultiCameraImages Multiimage)
{
	uWord_Type imgwidth;
	uWord_Type imgheight;
	uWord_Type imglen;

	uWord_Type stConfCheck[ACTIVE_WINDOW_MAX] =
	{
		uw_window1_showmode,
		uw_window2_showmode,
		uw_window3_showmode,
		uw_window4_showmode,
		uw_window5_showmode,
		uw_window7_showmode,
	};

	uWord_Type i = 0;

	//std::string cam_image;

	for (i = 0; i < ACTIVE_WINDOW_MAX; i++)
	{
		if (stConfCheck[i] == CONF_WINDOW_SHOW_ACT)
		{
			imgwidth = Multiimage.images[i].width;
			imgheight = Multiimage.images[i].height;
			imglen = Multiimage.images[i].pixels.length();

			if (checkMultiImagesize(imgwidth, imgheight, winname_st[i]))
			{
				cv::Mat m_inimagebuf = cv::Mat(cv::Size(imgwidth, imgheight), CV_8UC3);
				memcpy(m_inimagebuf.data, (void *)&(Multiimage.images[i].pixels[0]), imglen);
				cv::namedWindow(winname_st[i], cv::WINDOW_NORMAL);
				cv::imshow(winname_st[i], m_inimagebuf);
			}
		}
	}
}

/*!
* Multi Camera Image data size check
*/
uWord_Type ImageViewer::checkMultiImagesize(uWord_Type imgwidth, uWord_Type imgheight, std::string cam_image)
{
	uWord_Type retVal;
	retVal = false;

	if ((imgwidth == 0) || (imgheight == 0))
	{
		std::cerr << "[" << cam_image << "]" << "input CameraImage data is empty!" << std::endl;
	}
	else if ((imgwidth <= IMAGE_SIZE_W_MAX) && (imgheight <= IMAGE_SIZE_H_MAX))
	{
		retVal = true;
	}
	else
	{
		std::cerr << "[" << cam_image << "]" << "input CameraImage data is oversized!" << std::endl;
	}
	return retVal;
}

extern "C"
{
 
  void ImageViewerInit(RTC::Manager* manager)
  {
    coil::Properties profile(imageviewer_spec);
    manager->registerFactory(profile,
                             RTC::Create<ImageViewer>,
                             RTC::Delete<ImageViewer>);
  }
  
};

